function [adj, names, name_hash, contig, overlap] = add_ref_nodes_to_adj(adj, names, name_hash, contig, overlap, contig_ref, overlap_ref, name_hash_ref, pairs, distance)
    % Add edges to the adjency matrix    
     
    % Nothing to do 
    if isempty(pairs)
    	return
    end
        
    fprintf('Adding new nodes to the adjency\n');
    [adj, names, name_hash, contig] = add_nodes(adj, names, name_hash, contig, contig_ref);
    fprintf('Adding reference nodes to the adjency\n');
    [adj, names, name_hash, contig] = add_ref_nodes(adj, names, name_hash, contig, contig_ref);
    [adj, name_hash, overlap]       = add_ref_edges(adj, name_hash, overlap, overlap_ref, distance);
    
   

    function [adj, name_hash, overlap] = add_ref_edges(adj, name_hash, overlap, overlap_ref, distance)
        
	% Initialize overlap matrix for 'pairs'
    if size(overlap,1) > 0
        ov_new = repmat(overlap(1),1,size(pairs,1)*2);
    else
        ov_new = repmat(overlap,1,size(pairs,1)*2);
    end
        ov_pos = 1;

        OverlapQpos = make_hash({overlap_ref.Q});
        
        fprintf('Adding %d edges to the adjency matrix\n',size(pairs,1));
        fprintf('1000 nodes per dot are processed\n [');
        for ii = 1:size(pairs,1)
            
            if mod(ii,1000) == 0
                fprintf('.');
            end 
            
            ref_node_name = ['refnode_' num2str(pairs{ii,3}) '_' num2str(pairs{ii,4})];
            
            % Add overlap with reference node for first of pair
            Q_node        = pairs{ii,1};
            R_node        = ref_node_name;
            adj(name_hash(Q_node),name_hash(R_node)) = distance;
            
            align_start   = 1;
            align_end     = pairs{ii,3}-pairs{ii,5}; % TODO hier moet toch +1 bij??
            
            
            %loc          = arrayfun(@(x) strcmp(x.Q,Q_node),overlap_ref);
            loc               = OverlapQpos(Q_node);
            ov_new(ov_pos)    = overlap_ref(loc);
            ov_new(ov_pos).R  = R_node;
            ov_new(ov_pos).S1 = align_start;
            ov_new(ov_pos).E1 = align_end;
            ov_pos            = ov_pos + 1;
            
            
            % Add overlap with reference node for second of pair
            Q_node = pairs{ii,2};
            adj(name_hash(R_node),name_hash(Q_node)) = distance;
            
            align_start   = pairs{ii,4}-pairs{ii,5}; % TODO hier moet toch +1 bij??
            align_end     = pairs{ii,6}-pairs{ii,5}; % TODO hier moet toch +1 bij??
            
            
            %loc                = arrayfun(@(x) strcmp(x.Q,Q_node),overlap_ref);
            loc                 = OverlapQpos(Q_node);
            ov_new(ov_pos)      = overlap_ref(loc);
            ov_new(ov_pos).R    = R_node;
            ov_new(ov_pos).S1   = align_start;
            ov_new(ov_pos).E1   = align_end;
            ov_pos              = ov_pos + 1;
                        
        end
        
         overlap = [overlap ov_new];
         fprintf('] done\n');
        
    end
        
        
        

    function [adj, names, name_hash, contig] = add_nodes(adj, names, name_hash, contig, contig_ref)
        
        % varargin is the node_size for the reference nodes
     
        % Filter contigs
        [uNames,ui]          = unique({contig_ref.name});
        contig_ref           = contig_ref(ui);
        ref_contigs          = arrayfun(@(x) isequal(x.assembly,'reference'), contig_ref);
        contig_ref           = contig_ref(~ref_contigs);
	
        AlignedContigs       = {contig_ref.name};
        NotInAdj             = ~ismember(AlignedContigs,names);
        contig_NotInAdj      = contig_ref(NotInAdj);
        NrNotInAdj           = sum(NotInAdj);
        SizeAdj              = size(adj,1);
        NewSize              = SizeAdj + NrNotInAdj;
        adj(NewSize,NewSize) = 0;
        contig               = [contig; contig_NotInAdj];
        names                = [names; {contig_NotInAdj.name}'];
        
        % Add to name_hash
        pos = SizeAdj+1;
        for ni = 1:NrNotInAdj
            name_hash(contig_NotInAdj(ni).name) = pos;
            pos = pos+1;
        end
    end

    function [adj, names, name_hash, contig] = add_ref_nodes(adj, names, name_hash, contig, contig_ref)
               
        
        ref_chr_contig = find(arrayfun(@(x) isequal(x.assembly,'reference'), contig_ref));
        % contig_ref(1) is the reference chromosome
        contigR = contig_ref(ref_chr_contig);

        if length(ref_chr_contig) > 1
            error('multiple reference contigs (chromosomes) not allowed');
        end

        contigsR = repmat(contigR, size(pairs,1) ,1);

        for ii = 1:size(pairs,1)
            node_size         = abs(pairs{ii,6}-pairs{ii,5}); % TODO Hier moet toch -1 bij??
            node_name         = ['refnode_' num2str(pairs{ii,3}) '_' num2str(pairs{ii,4})];
            contigsR(ii).size = node_size;
            contigsR(ii).name = node_name;
        end
           
        NotInAdj             = ~ismember({contigsR.name},names);
        contigsR             = contigsR(NotInAdj);	
        NrNotInAdj           = length(contigsR);
        SizeAdj              = size(adj,1);
        NewSize              = SizeAdj + NrNotInAdj;
        adj(NewSize,NewSize) = 0;
        contig               = [contig; contigsR];
        names                = [names; {contigsR.name}'];
        
        % Add to name_hash
        pos = SizeAdj+1;
        for ni = 1:NrNotInAdj
            name_hash(contigsR(ni).name) = pos;
            pos = pos+1;
        end                            
        
    end
end
