function [nuc, del] = align_assemblies(assembly_files, assembly_names, mapfilter, alignlen, ident)
% Use nucmer to detect overlapping contigs in different assemblies
%
% INPUT 
% assmebly_files: cell array with location of multiple fasta files with contigs
% assembly_names: cell array with names of the assembly (e.g. Velvet,..)


nr_of_assemblies = length(assembly_files);

for i = 1:nr_of_assemblies-1
    for j = i+1:nr_of_assemblies
        
        % Assembly names
        rn = assembly_names{i};
        qn = assembly_names{j};
        
        % Assembly files
        rf = assembly_files{i};
        qf = assembly_files{j};
        
 
        [s,w] = unix(['nucmer --prefix=' rn '_' qn ' ' rf ' ' qf]);
        if s ~= 0, w,error('Nucmer failed or is not findable by matlab'), end;
        
        % Filter alignments
        [s,w] = unix(['delta-filter ' mapfilter ' -l ' num2str(alignlen) ' -i ' num2str(ident) ' ' rn '_' qn '.delta > ' rn '_' qn '_filtered.delta']);
        if s ~= 0, w,error('delta-filter failed or is not findable by matlab'), end;


        % Make a human readable table
        % %%% [s,w] = unix(['show-coords -rcl ' rn '_' qn '_filtered.delta > ' rn '_' qn '.coords']);
        % %%% if s ~= 0, w,error('show-coords failed or is not findable by matlab'), end;
        
        % Filter contigs that align with an end to the reference
        %[s,w] = unix(['/Volumes/linux-bastion/perl/filter_reference_alignments.pl out.coords ' num2str(clipping_thrs) ' > out.coords_filtered']);
        % Get filename


        % %%% coords_file = [pwd '/' rn '_' qn '.coords'];
        delta_file = [pwd '/' rn '_' qn '_filtered.delta'];
        % Read nucmer files
        [del m] = readdelta(delta_file);
        
        if ~isempty(m)
            del = merge_alignment(del,'delta',m);
        end
        % %%% nuc = readnucmer({coords_file});
        % %%% nuc = merge_alignment(nuc,'coords');       
        nuc = [];
    end
end

