function[G, contig] = direct_graph(G, overlap, contig, names, name_hash, clipping_thrs, startnode, priority_nodes, all_sub_graphs, ref_first)
% Direct a adjency graph

if nargin < 10
    % If true, reference nodes will have priority in orienting the contigs
    % This in combination with a 'tiling' used for reference nodes will 
    % garantee a solution
    ref_first = true;
    if nargin < 8
        if nargin < 7
            priority_nodes = [];
        else
            priority_nodes = startnode;
        end
    
    end
end

% Convert overlap struct to cell array
ov = struct2cell(overlap);
ov = squeeze(ov(1:2,:,:));


tic
Rhash = make_hash_multiple_positions({overlap.R});
Qhash = make_hash_multiple_positions({overlap.Q});
toc

% Assign an orientation to each contig
fprintf('Determine contig orientation\n');
%[G, contig, subgraph_nodes] = orient_subgraphs(G, overlap, contig, names, ov, startnode, [],          [], priority_nodes, all_sub_graphs);
%                              orient_subgraphs(G, overlap, contig, names, ov, startnode, previousnode, P, priority_nodes, all_sub_graphs)
        % Loop through all subgraphs to orient the contigs
        
previousnode = [];
P = [];

tic
if  all_sub_graphs == true    
    % Orient all subgraphs
    startnodes     = [priority_nodes setdiff(1:size(G,1),priority_nodes)];
else
    % Just orient startnode and priority node subgraphs
    startnodes     = [setdiff(startnode,priority_nodes) priority_nodes ];
end

NodesProcessed = 0; % counter
fprintf('1000 edges of %d processed per dot [',size(G,1));

for inode = 1:length(startnodes)

    % Get next node from priority queue
    startnode = startnodes(inode);
    
    % If node has already been oriented, continue
    if sum(P == startnode) > 0
        continue
    end
    
    % Orient contigs starting at node 'startnode'
    %[G, contig, P, NodesProcessed] = orient_contigs(startnode, G, overlap, contig, names, ov,  previousnode, P, NodesProcessed);
    orient_contigs(startnode,previousnode);
end 

subgraph_nodes = make_hash(names(P));   
fprintf('] done\n');
toc


% Remove edges that do not agree with the orientation
fprintf('Direct edges\n');
if all_sub_graphs == true
    G = make_directed(G, contig, overlap, name_hash, clipping_thrs);
else
    G = make_directed(G, contig, overlap, name_hash, clipping_thrs, subgraph_nodes);
end

    function G = make_directed(G, contig, overlap, name_hash, clipping_thrs, subgraph_nodes)
    % Remove edges that contradict orientation 
    
        for oi = 1:length(overlap)
            
            % If only a subgraph has been oriented, the corresponding nodes
            % are stored in 'subgraph_nodes'. 
            if nargin > 5
                if ~(isKey(subgraph_nodes, overlap(oi).R) && isKey(subgraph_nodes, overlap(oi).Q))
                    continue
                end
            end


            at = alignment_type(contig,overlap(oi),name_hash);
            if at == -1
                error(['No alignment type found for' overlap(oi)]);                
            elseif at == 33 || at == 55
                [Rcontained Qcontained] = iscontained(overlap(oi), clipping_thrs);
                
                % If both contig are contained, remove the edges. If only
                % on is contained, the edge of the uncontained contig will
                % be leading in the direction
                if (Rcontained && Qcontained)
                    G = remove_edge(G,name_hash,overlap(oi).R, overlap(oi).Q);
                    G = remove_edge(G,name_hash,overlap(oi).Q, overlap(oi).R);
                elseif Qcontained
                    if at == 33
                        G = remove_edge(G,name_hash,overlap(oi).Q, overlap(oi).R);
                    else
                        G = remove_edge(G,name_hash,overlap(oi).R, overlap(oi).Q);
                    end
                elseif Rcontained
                    if at == 33
                        G = remove_edge(G,name_hash,overlap(oi).R, overlap(oi).Q);
                    else
                        G = remove_edge(G,name_hash,overlap(oi).Q, overlap(oi).R);
                    end
                else
                    G = remove_edge(G,name_hash,overlap(oi).R, overlap(oi).Q);
                    G = remove_edge(G,name_hash,overlap(oi).Q, overlap(oi).R);
                end
                    
            elseif at == 53
                G = remove_edge(G,name_hash,overlap(oi).R, overlap(oi).Q);
            elseif at == 35
                G = remove_edge(G,name_hash,overlap(oi).Q, overlap(oi).R);
            end     
        end
        
    end
    

    function type = alignment_type(contig,overlap,name_hash)
        
        locR = name_hash(overlap.R);
        locQ = name_hash(overlap.Q);
                
        [aeR aeQ] = aligned_end(contig,overlap,name_hash);

                
        if isequal(aeR,'left') && isequal(aeQ,'left')
            if (contig(locR).orient == 53) && (contig(locQ).orient == 53)
                type = 55;
            elseif (contig(locR).orient == 53) && contig(locQ).orient == 35
                type = 53;
            elseif (contig(locR).orient == 35) && contig(locQ).orient == 53
                type = 35;
            elseif (contig(locR).orient == 35) && contig(locQ).orient == 35
                type = 33;
            else
                type = -1;
            end
            
        elseif isequal(aeR,'left') && isequal(aeQ,'right')
            if (contig(locR).orient == 53) && (contig(locQ).orient == 53)
                type = 53;
            elseif (contig(locR).orient == 53) && contig(locQ).orient == 35
                type = 55;
            elseif (contig(locR).orient == 35) && contig(locQ).orient == 53
                type = 33;
            elseif (contig(locR).orient == 35) && contig(locQ).orient == 35
                type = 35;
            else
                type = -1;
            end
        elseif isequal(aeR,'right') && isequal(aeQ,'left')
            if (contig(locR).orient == 53) && (contig(locQ).orient == 53)
                type = 35;
            elseif (contig(locR).orient == 53) && contig(locQ).orient == 35
                type = 33;
            elseif (contig(locR).orient == 35) && contig(locQ).orient == 53
                type = 55;
            elseif (contig(locR).orient == 35) && contig(locQ).orient == 35
                type = 53;
            else
                type = -1;
            end            
        elseif isequal(aeR,'right') && isequal(aeQ,'right')
            if (contig(locR).orient == 53) && (contig(locQ).orient == 53)
                type = 33;
            elseif (contig(locR).orient == 53) && contig(locQ).orient == 35
                type = 35;
            elseif (contig(locR).orient == 35) && contig(locQ).orient == 53
                type = 53;
            elseif (contig(locR).orient == 35) && contig(locQ).orient == 35
                type = 55;
            else
                type = -1;
            end          
        else
            type = -1;
        end
    end
    

    function [aeR aeQ] = aligned_end(contig,overlap,name_hash) 
        
        locR = name_hash(overlap.R);
        locQ = name_hash(overlap.Q);
                
        % Not rev comp is used, but normal one
        if overlap.E1 > overlap.S1
            if overlap.S1 < contig(locR).size - overlap.E1
                aeR = 'left';
            else
                aeR = 'right';
            end
        else
            if overlap.E1 < contig(locR).size - overlap.S1
                aeR = 'left';
            else
                aeR = 'right';
            end
        end
        
        if isequal(contig(locR).assembly,'reference')
            % Not rev comp is used, but normal one
            if overlap.E2 > overlap.S2
                switch aeR
                    case 'left'
                        aeQ = 'right';
                    case 'right'
                        aeQ = 'left';
                end
            else
                switch aeR
                    case 'right'
                        aeQ = 'right';
                    case 'left'
                        aeQ = 'left';
                end
                
            end
        else      
        
            % Not rev comp is used, but normal one
            if overlap.E2 > overlap.S2
                if overlap.S2 < contig(locQ).size - overlap.E2
                    aeQ = 'left';
                else 
                    aeQ = 'right';
                end
            else
                if overlap.E2 < contig(locQ).size - overlap.S2
                    aeQ = 'left';
                else
                    aeQ = 'right';
                end
            end
        end

    end
    

    function G = remove_edge(G,name_hash,contig1, contig2)
        % Remove edge (directed) from graph
        loc1 = name_hash(contig1);
        loc2 = name_hash(contig2);
        G(loc1,loc2) = 0;
    end


%{
    function [G, contig, subgraph_nodes] = orient_subgraphs(G, overlap, contig, names, ov, startnode, previousnode, P, priority_nodes, all_sub_graphs)
        % Loop through all subgraphs to orient the contigs
        
        tic
        if  all_sub_graphs == true    
            % Orient all subgraphs
            startnodes     = [priority_nodes setdiff(1:size(G,1),priority_nodes)];
        else
            % Just orient startnode and priority node subgraphs
            startnodes     = [setdiff(startnode,priority_nodes) priority_nodes ];
        end
            
        NodesProcessed = 0; % counter
        fprintf('1000 edges of %d processed per dot [',size(G,1));
        
        for inode = 1:length(startnodes)
            
            startnode = startnodes(inode);
            
            if sum(P == startnode) > 0
                continue
            end
            
            % Orient contigs starting at node 'startnode'
            %[G, contig, P, NodesProcessed] = orient_contigs(startnode, G, overlap, contig, names, ov,  previousnode, P, NodesProcessed);
            orient_contigs(startnode);
        end 
        
        subgraph_nodes = make_hash(names(P));   
        fprintf('] done\n');
        toc
    end
%}

    %function [G, contig, P, NodesProcessed] = orient_contigs(startnode, G, overlap, contig, names, ov, previousnode, P, NodesProcessed)
    function orient_contigs(startnode, previousnode)
    % Add orientation to the contigs
    % Orientation starts with assigning a random orientation to a contig
    % and subsequently greedily assigning orientations to all connected
    % contigs. This procedure has to be done for all connected subgraphs
    %
    % No arguments are given to this function memory usage in deep recursion
    % instead 'shared variables' are used from the parent function ('orient_subgraphs')
        
	NodesProcessed = NodesProcessed + 1;
        if mod(NodesProcessed,1000) == 0
            fprintf('.');
            if mod(NodesProcessed,50000) == 0
                fprintf('\n');
            end
        end
    
        % Nodes that we've oriented already
        P = [P startnode];          

        if ~isempty(previousnode)
            % Check whether the overlap is reversal or not (see BAMBUS paper for definition)
            reversal = isreversal(startnode,previousnode,names,overlap,ov);

            % Assignment of orientation
            if ~reversal
                if contig(previousnode).orient == 53
                    contig(startnode).orient = check_orientation(contig(startnode), 53);
                elseif contig(previousnode).orient == 35
                    contig(startnode).orient = check_orientation(contig(startnode), 35);
                end
            elseif reversal
                if contig(previousnode).orient == 53
                    contig(startnode).orient = check_orientation(contig(startnode), 35);
                elseif contig(previousnode).orient == 35
                    contig(startnode).orient = check_orientation(contig(startnode), 53);
                end
            end
            
        else
            % If it's the first node, then just assign something random, or
            % use the orientation obtained from the tiling, if available
            if isfield(contig(startnode),'preferred_orient')
                if contig(startnode).preferred_orient == 53
                    contig(startnode).orient = 53;
                elseif contig(startnode).preferred_orient == 35
                    contig(startnode).orient = 35;
                else
                    contig(startnode).orient = 53;
                end
            else
                contig(startnode).orient = 53;
            end
        end

        % Find the nodes connected to begin
        nodes_connected = find(logical(G(startnode,:)));  
        nodes_connected_weights = G(startnode,nodes_connected);
        [sorted_w, index] = sort(nodes_connected_weights,'descend');
        nodes_connected = nodes_connected(index);

        % If reference nodes are being visited first to ensure a solution,
        % redo the ordering such that reference nodes appear first in line
        if ref_first
            ref_node_indices = ismember({contig(nodes_connected).assembly},'reference');
            nodes_connected = [nodes_connected(ref_node_indices) nodes_connected(~ref_node_indices)];
        end
        
        % Now go to all connected nodes of startnode, that do not have an
        % orientation already (i.e. appear in P)
        for i = 1:length(nodes_connected)          % Extend to all nodes connected to begin
            next_node = nodes_connected(i);        % Extend to node i
            if sum(next_node == P) == 0            % If the node i is in the path, the skip to prevent cycles
                orient_contigs(next_node,startnode);
                %[G, contig, P, NodesProcessed] = orient_contigs(startnode, G, overlap, contig, names, ov, next_node, P, NodesProcessed);  % Find path from next_node
            end
        end
    end

    function orientation = check_orientation(SingleContig, orientation)
        % Assign orientation to a contig
        if (SingleContig.preferred_orient == 53 && orientation == 35) || (SingleContig.preferred_orient == 35 && orientation == 53)
            warning(['Contig ' SingleContig.name ' has been assigned a different orientation compared to the alignment to the reference.']);
        end
    end

    
    function rev = isreversal(startnode,previousnode,names,overlap,ov)
        % Determin reversal edges
        
        ovi = find_overlap_pos(names{startnode}, names{previousnode});
        
        if isempty(ovi)
            error(['No overlap found for node ' names{startnode} ' and ' names{previousnode}]);
        end
        
        % Check for reversal edge
        if overlap(ovi).S1 < overlap(ovi).E1
            if overlap(ovi).S2 < overlap(ovi).E2
                rev = false;
            elseif overlap(ovi).S2 > overlap(ovi).E2
                rev = true;
            end
        elseif overlap(ovi).S1 > overlap(ovi).E1
            if overlap(ovi).S2 > overlap(ovi).E2
                rev = false;
            elseif overlap(ovi).S2 < overlap(ovi).E2
                rev = true;
            end
        end
        
    end
    
    function ovi = find_overlap_pos(contig1, contig2)
        % return overlap index for two contig names        
        
        if isKey(Rhash, contig1) && isKey(Qhash, contig2)
            ovi = intersect(Rhash(contig1),Qhash(contig2));
        else
            ovi = [];
        end

        if isempty(ovi)
            if isKey(Rhash, contig2) && isKey(Qhash, contig1)
               ovi = intersect(Rhash(contig2),Qhash(contig1));
            end
        end
    end


%{    
    function ovi = find_overlap_pos(ov, contig1, contig2)
        % return overlap index for two contig names        
        
        posR1 = ismember(ov(2,:),contig1);
        posQ1 = ismember(ov(1,:),contig2);
        
        if sum(posR1.*posQ1) == 0
            posR1 = ismember(ov(2,:),contig2);
            posQ1 = ismember(ov(1,:),contig1);
        end
        
        % position in overlap list
        ovi = find(posR1.*posQ1);
    end
%}        
        
end


