% Exports an xgmml file based on the specified nodes and edges
%
%  INPUT
%     filename    Name of the file to output
%     graphname   Name of the graph in cytoscape
%     nodes       Struct array containing structs representing nodes
%                 node.id = id number, referenced from edges
%                 node.label = label, displayed in graph
%                 node.att.x = attribute x, can be browsed through in the node attribute browser
%                 node.graphics.[borderwidth,width,heigh,color,type,etc.] = various fields specifying the graphical representation
%     edges       Struct array containg structs representing edges
%                 edge.source = node id
%                 edge.target = node id
%                 edge.att.x = attribute x, can be browsed through in the edge attribute browser
%                 edge.graphics.[borderwidth,color,sourceArrow,targetArrow,etc.] = various fields specifiying the graphical representation
%
%    NOTE
%     For possible attributes/values for the graphics fields, see JavaDoc documentation Cytoscape. 


function export_xgmml(filename,graphname,nodes,edges)

f = fopen(filename,'w');
fprintf(f,'<?xml version=\"1.0\"?>\n');
fprintf(f,'<graph id=\"%s\" label=\"%s\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:ns1=\"http://www.w3.org/1999/xlink\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns=\"http://www.cs.rpi.edu/XGMML\">\n',graphname,graphname);
fprintf(f,'<att name=\"documentVersion\" value=\"1.0\" />\n');
fprintf(f,'<att label=\"documentVersion\" name=\"documentVersion\" value=\"1.0\" type=\"string\"/>\n');

for i = 1:length(nodes)
   cn = nodes(i);
   fprintf(f,'<node id=\"%d\" label=\"%s\" name=\"base\">\n',cn.id,cn.label);
   if(isfield(cn,'att'))
      cna = cn.att;
      cna_fieldnames = fieldnames(cna);
      for j = 1:length(cna_fieldnames)
         k = getfield(cna,cna_fieldnames{j});
         if(iscell(k))
            fprintf(f','\t<att label=\"%s_list\" name=\"%s_list\" type=\"list\" >\n',cna_fieldnames{j},cna_fieldnames{j});
               for y = 1:length(k)
                  if(isnumeric(k{y}))
                     fprintf(f,'\t\t<att value=\"%d\" type=\"real\" />\n',k{y});
                  else
                     fprintf(f,'\t\t<att value=\"%s\" type=\"string\" />\n',special_chars(k{y}));
                  end;
               end;
            fprintf(f,'\t</att>\n')
         else
            if(isnumeric(k))
                fprintf(f','\t<att label=\"%s\" name=\"%s\" value=\"%d\" type=\"real\" />\n',cna_fieldnames{j},cna_fieldnames{j},k);
            else
                fprintf(f','\t<att label=\"%s\" name=\"%s\" value=\"%s\" type=\"string\" />\n',cna_fieldnames{j},cna_fieldnames{j},special_chars(k));
            end;
         end;
      end;
   end;
   if(isfield(cn,'graphics'))
      cng = cn.graphics;
      if(~isfield(cng,'borderwidth'))
         cng.borderwidth = 1;
      end;
      if(~isfield(cng,'color'))
         cng.color = '#0000FF';
      end;
      if(~isfield(cng,'height'))
         cng.height = 40;
      end;
      if(~isfield(cng,'width'))
         cng.width = 40;
      end;
      if(~isfield(cng,'type'))
         cng.type = 'circle';
      end;
      fprintf(f','\t<graphics type=\"%s\" width=\"%d\" fill=\"%s\" x="1" y="1" w="%d" h="%d">\n',cng.type,cng.borderwidth,cng.color,cng.height,cng.width);
      cng = rmfield(cng,'width');
      cng = rmfield(cng,'height');
      cng = rmfield(cng,'borderwidth');
      cng = rmfield(cng,'color');
      cng = rmfield(cng,'type');
      cng_fieldnames = fieldnames(cng);
      if(length(cng_fieldnames) > 0)
         fprintf(f,'\t\t<att name=\"cytoscapeNodeGraphicsAttributes\">\n');
      end;
      for j = 1:length(cng_fieldnames)
         if(isnumeric(getfield(cng,cng_fieldnames{j})))
            fprintf(f','\t\t\t<att name=\"%s\" value=\"%d\"/>\n',cng_fieldnames{j},getfield(cng,cng_fieldnames{j}));
         else
            fprintf(f','\t\t\t<att name=\"%s\" value=\"%s\"/>\n',cng_fieldnames{j},special_chars(getfield(cng,cng_fieldnames{j})));
         end;
      end;
      if(length(cng_fieldnames) > 0)
         fprintf(f,'\t\t</att>\n');
      end;
      fprintf(f,'\t</graphics>\n');
   end;
   fprintf(f,'</node>\n');
end;

for i = 1:length(edges)
   ce = edges(i);
   if(isfield(ce, 'label'))
       fprintf(f,'<edge source=\"%d\" target=\"%d\" label=\"%s\" id=\"%d\">\n',ce.source,ce.target,ce.label,i);
   else
       fprintf(f,'<edge source=\"%d\" target=\"%d\" id=\"%d\">\n',ce.source,ce.target,i);
   end;
   if(isfield(ce,'att'))
      cea = ce.att;
      cea_fieldnames = fieldnames(cea);
      for j = 1:length(cea_fieldnames)
         if(isnumeric(getfield(cea,cea_fieldnames{j})))
           fprintf(f','\t<att label=\"%s\" name=\"%s\" value=\"%d\" type=\"string\"/>\n',cea_fieldnames{j},cea_fieldnames{j},getfield(cea,cea_fieldnames{j}));
         else
           fprintf(f','\t<att label=\"%s\" name=\"%s\" value=\"%s\" type=\"string\"/>\n',cea_fieldnames{j},cea_fieldnames{j},special_chars(getfield(cea,cea_fieldnames{j})));
         end;
      end;
   end;
   if(isfield(ce,'graphics'))
      ceg = ce.graphics;
      if(~isfield(ceg,'borderwidth'))
         ceg.borderwidth = 1;
      end;
      if(~isfield(ceg,'color'))
         ceg.color = '#0000FF';
      end;
      fprintf(f','\t<graphics width=\"%d\" fill=\"%s\">\n',round(ceg.borderwidth),ceg.color);
      ceg = rmfield(ceg,'borderwidth');
      ceg = rmfield(ceg,'color');
      ceg_fieldnames = fieldnames(ceg);
      if(length(ceg_fieldnames) > 0)
         fprintf(f,'\t\t<att name=\"cytoscapeEdgeGraphicsAttributes\">\n');
      end;
      for j = 1:length(ceg_fieldnames)
         if(isnumeric(getfield(ceg,ceg_fieldnames{j})))
            fprintf(f','\t\t\t<att name=\"%s\" value=\"%d\"/>\n',ceg_fieldnames{j},getfield(ceg,ceg_fieldnames{j}));
         else
            fprintf(f','\t\t\t<att name=\"%s\" value=\"%s\"/>\n',ceg_fieldnames{j},special_chars(getfield(ceg,ceg_fieldnames{j})));
         end;
      end;
      if(length(ceg_fieldnames) > 0)
         fprintf(f,'\t\t</att>\n');
      end;
      fprintf(f,'\t</graphics>\n');
   end;
   fprintf(f,'</edge>\n');
end;

fprintf(f,'</graph>\n');
fclose(f);






