function [chr_nr contig_nr] = getchromosomenr(names,name_type)
%GETCHROMOSOMENR returns chromosome and contig numbers
%
% of a list of contig names which are in the form of: chr10_contig_237
%
% INPUT:
% NAMES:        cell array with contig names
% NAME_TYPE (optional): Several type of names have been implemented
%                       'standard': chr11_contig_234
%                       'roman'   : chrXI_contig_234
%                       'no15'    : same as standard but 'gi' for chromosome 15 (typical for my dataset ;)  
%
% Currently for all options the mitochondrial chromosome input is 'mt' and output is 17 
%
%
%
% Author: Jurgen Nijkamp
% Date  : 2 September 2009


if nargin < 2
    name_type = 'standard';
end

if isequal(name_type,'standard')
    
    contig_nr = zeros(size(names,1),1);
    chr = regexp(names,'_','split');
    chr_nr = zeros(size(chr,1),1);
    for i = 1:size(chr,1)
        contig_nr(i) = str2num(chr{i}{3});
        chr{i} = chr{i}{1};
        switch chr{i}
            case 'chr1'
                chr_nr(i) = 1;
            case 'chr10'
                chr_nr(i) = 10;
            case 'chr11'
                chr_nr(i) = 11;
            case 'chr12'
                chr_nr(i) = 12;
            case 'chr13'
                chr_nr(i) = 13;
            case 'chr14'
                chr_nr(i) = 14;
            case 'chr16'
                chr_nr(i) = 16;
            case 'chr2'
                chr_nr(i) = 2;
            case 'chr3'
                chr_nr(i) = 3;
            case 'chr4'
                chr_nr(i) = 4;
            case 'chr5'
                chr_nr(i) = 5;
            case 'chr6'
                chr_nr(i) = 6;
            case 'chr7'
                chr_nr(i) = 7;
            case 'chr8'
                chr_nr(i) = 8;
            case 'chr9'
                chr_nr(i) = 9;
            case 'chr15'
                chr_nr(i) = 15;
            case 'mt'
                chr_nr(i) = 17;
            otherwise
                error('query sequence chromosome number not found')

        end
    end

elseif isequal(name_type,'roman')

    % Get contig and chromosome numbers
    contig_nr = zeros(size(names,1),1);
    chr = regexp(names,'_','split');
    chr_nr = zeros(size(chr,1),1);


    for i = 1:size(chr,1)
        contig_nr(i) = str2num(chr{i}{3});
        chr{i} = chr{i}{1};
        switch chr{i}
            case 'chrI'
                chr_nr(i) = 1;
            case 'chrX'
                chr_nr(i) = 10;     
            case 'chrXI'
                chr_nr(i) = 11;
            case 'chrXII'
                chr_nr(i) = 12;
            case 'chrXIII'
                chr_nr(i) = 13;
            case 'chrXIV'
                chr_nr(i) = 14;
            case 'chrXVI'
                chr_nr(i) = 16;
            case 'chrII'
                chr_nr(i) = 2;
            case 'chrIII'
                chr_nr(i) = 3;
            case 'chrIV'
                chr_nr(i) = 4;
            case 'chrV'
                chr_nr(i) = 5;
            case 'chrVI'
                chr_nr(i) = 6;
            case 'chrVII'
                chr_nr(i) = 7;
            case 'chrVIII'
                chr_nr(i) = 8;
            case 'chrIX'
                chr_nr(i) = 9;
            case 'chrXV'
                chr_nr(i) = 15;
            case 'mt'                % ??? is er niet..
                chr_nr(i) = 17;
            otherwise
                error(['qry sequence chromosome ' chr{i} ' not found in switch'])

        end %switch 
    end %for
    


elseif isequal(name_type,'no15')
    
    contig_nr = zeros(size(names,1),1);
    chr = regexp(names,'_','split');
    chr_nr = zeros(size(chr,1),1);
    for i = 1:size(chr,1)
        contig_nr(i) = str2num(chr{i}{3});
        chr{i} = chr{i}{1};
        switch chr{i}
            case 'chr1'
                chr_nr(i) = 1;
            case 'chr10'
                chr_nr(i) = 10;
            case 'chr11'
                chr_nr(i) = 11;
            case 'chr12'
                chr_nr(i) = 12;
            case 'chr13'
                chr_nr(i) = 13;
            case 'chr14'
                chr_nr(i) = 14;
            case 'chr16'
                chr_nr(i) = 16;
            case 'chr2'
                chr_nr(i) = 2;
            case 'chr3'
                chr_nr(i) = 3;
            case 'chr4'
                chr_nr(i) = 4;
            case 'chr5'
                chr_nr(i) = 5;
            case 'chr6'
                chr_nr(i) = 6;
            case 'chr7'
                chr_nr(i) = 7;
            case 'chr8'
                chr_nr(i) = 8;
            case 'chr9'
                chr_nr(i) = 9;
            case 'gi'
                chr_nr(i) = 15;
            case 'mt'
                chr_nr(i) = 17;
            otherwise
                error('query sequence chromosome number not found')

        end
    end


else
    error('unkown name_type');
end